@@ -1,3 +1,4 @@
+import time
 import json
 import logging
 import requests
@@ -45,7 +46,7 @@ class Implementation(Imp):
     '''
 
     @BaseConnection.locked
-    def connect(self, timeout=30):
+    def connect(self, timeout=30, retries=3, retry_wait=10):
         '''connect to the device via REST
 
         Arguments
@@ -53,6 +54,10 @@ class Implementation(Imp):
 
             timeout (int): Timeout value
 
+            retries (int): Max retries on request exception (default: 3)
+
+            retry_wait (int): Seconds to wait before retry (default: 10)
+
         Raises
         ------
 
@@ -125,19 +130,30 @@ class Implementation(Imp):
         self.session = requests.Session()
         _data = json.dumps(payload)
 
-        # Connect to the device via requests
-        response = self.session.post(login_url, data=_data, timeout=timeout,
-                                     verify=False, headers=headers)
-        log.info(response)
+        for _ in range(retries):
+            try:
+                # Connect to the device via requests
+                response = self.session.post(login_url, data=_data, timeout=timeout,
+                                             verify=False, headers=headers)
+                log.info(response)
+
+                # Make sure it returned requests.codes.ok
+                if response.status_code != requests.codes.ok:
+                    log.error(response.text)
+                    # Something bad happened
+                    raise RequestException("Connection to '{ip}' has returned the "
+                                           "following code '{c}', instead of the "
+                                           "expected status code '{ok}'"
+                                           .format(ip=ip, c=response.status_code,
+                                                   ok=requests.codes.ok))
+                break
+            except Exception:
+                log.warning('Request to {} failed. Waiting {} seconds before retrying\n'.format(
+                             self.device.name, retry_wait), exc_info=True)
+                time.sleep(retry_wait)
+        else:
+            raise ConnectionError('Connection to {} failed'.format(self.device.name))
 
-        # Make sure it returned requests.codes.ok
-        if response.status_code != requests.codes.ok:
-            # Something bad happened
-            raise RequestException("Connection to '{ip}' has returned the "
-                                   "following code '{c}', instead of the "
-                                   "expected status code '{ok}'"
-                                   .format(ip=ip, c=response.status_code,
-                                           ok=requests.codes.ok))
         self._is_connected = True
         log.info("Connected successfully to '{d}'".format(d=self.device.name))
 
@@ -181,6 +197,7 @@ class Implementation(Imp):
     def get(self, dn, query_target='self', rsp_subtree='no', \
             query_target_filter='', rsp_prop_include='all', \
             rsp_subtree_include='', rsp_subtree_class='',\
+            target_subtree_class='',\
             expected_status_code=requests.codes.ok, timeout=30):
         '''GET REST Command to retrieve information from the device
 
@@ -208,6 +225,7 @@ class Implementation(Imp):
             rsp_subtree_include (string): specify additional contained objects 
                                           or options to be included
             rsp_subtree_class (string) : specify classes
+            target_subtree_class (string): specify subtree classes
             query_target_filter (string): filter expression
             expected_status_code (int): Expected result
         '''
@@ -236,6 +254,10 @@ class Implementation(Imp):
             full_url += "&rsp-subtree-class={rsc}"\
                 .format(rsc=rsp_subtree_class)
 
+        if target_subtree_class:
+            full_url += "&target-subtree-class={tsc}"\
+                .format(tsc=target_subtree_class)
+
         log.info("Sending GET command to '{d}':"\
                  "\nDN: {furl}".format(d=self.device.name, furl=full_url))
 
